
/* JavaScript content from spec/js/app/models/RefinementSpec.js in folder common */
define([
  "app/models/Refinement",
  "app/models/refinementsFactory"],
  function(Refinement, refinementsFactory) {
    describe("Refinement", function() {
      var refinement;

      beforeEach(function () {
        refinement = refinementsFactory.makeFakeData();
      });

      describe('refinementsList()', function () {
        it('returns a flat list of sub refinements', function () {
          expect(refinement.refinementsList()).toEqual([
            refinement.refinements[0][0],
            refinement.refinements[1][0],
            refinement.refinements[1][1],
            refinement.refinements[1][2]
          ]);
        });
      });
      describe('selectedSubRefinements()', function () {
        var expectedSelectedSubRefinements;

        beforeEach(function () {
          expectedSelectedSubRefinements = [];
          var refinementsList = refinement.refinementsList();

          refinementsList[0].selected = true;
          expectedSelectedSubRefinements.push(refinementsList[0]);

          refinementsList[1].selected = true;
          expectedSelectedSubRefinements.push(refinementsList[1]);
        });

        it('returns the subrefinements which are selected', function () {
          expect(refinement.selectedSubRefinements()).toEqual(expectedSelectedSubRefinements);
        });
      });
      describe('selectionDescription()', function () {
        describe('when the refinement has no sub selection', function () {
          it('returns "All"', function () {
            expect(refinement.selectionDescription()).toEqual('All');
          });
        });
        describe('when the refine has a sub selection', function () {
          describe('for Refinements named "Categories"', function () {
            var categories, selectedCategory;
            beforeEach(function () {
              categories = refinement.refinements[0][0];
              selectedCategory = categories.refinements[0][0];
              selectedCategory.selected = true;
            });
            it('returns the name of the selected category', function () {
              expect(categories.selectionDescription()).toBe(selectedCategory.name);
            });
          });
          describe('for Refinements named "Customer Rating"', function () {
            var customerRating;
            beforeEach(function () {
              customerRating = refinement.refinements[1][1];
              var ratings = customerRating.refinements[0];
              ratings[ratings.length-1].selected = true;
            });
            it('returns markup containing the correct number of star icons', function () {
              var span = customerRating.selectionDescription();
              expect(span.tagName).toEqual('SPAN');
              expect(span.childNodes.length).toBe(1);
            });
          });
          describe('for Refinements named "Price"', function () {
            var price;
            beforeEach(function () {
              price = refinement.refinements[1][0];
            });
            describe('when only a minimum is specified', function () {
              beforeEach(function () {
                price.set('range', { min: 20 });
              });
              it('returns a description of the lower bound', function () {
                expect(price.selectionDescription()).toEqual('£20 and up');
              });
            });
            describe('when only a maximum is specified', function () {
              beforeEach(function () {
                price.set('range', { max: 20 });
              });
              it('returns a description of the upper bound', function () {
                expect(price.selectionDescription()).toEqual('up to £20');
              });
            });
            describe('when only a range is specified', function () {
              beforeEach(function () {
                price.set('range', { min: 20, max: 30 });
              });
              it('returns a description of the range', function () {
                expect(price.selectionDescription()).toEqual('£20-30');
              });
            });
          });
          describe('for Refinements named "Brand"', function () {
            var brand;
            beforeEach(function () {
              brand = refinement.refinements[1][2];
            });
            describe('when one brand is selected', function () {
              var selectedBrand;
              beforeEach(function (){
                selectedBrand = brand.refinements[0][0];
                selectedBrand.selected = true;
              });
              it('returns the name of the brand', function () {
                expect(brand.selectionDescription()).toEqual(selectedBrand.name);
              });
            });
            describe('when more than one brand is selected', function () {
              beforeEach(function (){
                brand.refinements[0][0].selected = true;
                brand.refinements[0][1].selected = true;
              });
              it('returns the number of brands selected', function () {
                expect(brand.selectionDescription()).toEqual('2 selected');
              });
            });
          });
        });
      });
      describe('hasSubSelection()', function () {
        beforeEach(function () {
          refinement.refinements[0][0].selected = true;
        });
        it('returns true if there are sub selections', function () {
          expect(refinement.hasSubSelection()).toBeTruthy();
        });
      });
      describe('isLeaf()', function () {
        it('returns true if the refinement is a leaf node', function () {
          expect(refinement.refinements[0][0].refinements[0][0].isLeaf()).toBeTruthy();
        });
        it('returns false if the refinement is a leaf node', function () {
          expect(refinement.isLeaf()).toBeFalsy();
        });
      });

      describe('rclear()', function () {
        var expectedSelectedDescendantRefinements;
        beforeEach(function () {
          expectedSelectedDescendantRefinements = [];

          refinement.refinements[0][0].selected = true;
          expectedSelectedDescendantRefinements.push(refinement.refinements[0][0]);

          refinement.refinements[1][0].selected = true;
          expectedSelectedDescendantRefinements.push(refinement.refinements[1][0]);

          refinement.refinements[0][0].refinements[0][0].selected = true;
          expectedSelectedDescendantRefinements.push(refinement.refinements[0][0].refinements[0][0]);
        });
        it('recursively deselects refinements', function () {
          refinement.rclear();
          expectedSelectedDescendantRefinements.forEach(function (r) {
            expect(r.selected).toBe(false);
          });
        });
      });
    });
  }
);
